@@ -5,7 +5,7 @@
 
     Lexers for computer graphics and plotting related languages.
 
-    :copyright: Copyright 2006-2019 by the Pygments team, see AUTHORS.
+    :copyright: Copyright 2006-2021 by the Pygments team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -425,7 +425,7 @@ class AsymptoteLexer(RegexLexer):
         ],
         'statements': [
             # simple string (TeX friendly)
-            (r'"(\\\\|\\"|[^"])*"', String),
+            (r'"(\\\\|\\[^\\]|[^"\\])*"', String),
             # C style string (with character escapes)
             (r"'", String, 'string'),
             (r'(\d+\.\d*|\.\d+|\d+)[eE][+-]?\d+[lL]?', Number.Float),
@@ -775,7 +775,26 @@ class PovrayLexer(RegexLexer):
             (r'[0-9]+\.[0-9]*', Number.Float),
             (r'\.[0-9]+', Number.Float),
             (r'[0-9]+', Number.Integer),
-            (r'"(\\\\|\\"|[^"])*"', String),
+            (r'"(\\\\|\\[^\\]|[^"\\])*"', String),
             (r'\s+', Text),
         ]
     }
+
+    def analyse_text(text):
+        """POVRAY is similar to JSON/C, but the combination of camera and
+        light_source is probably not very likely elsewhere. HLSL or GLSL
+        are similar (GLSL even has #version), but they miss #declare, and
+        light_source/camera are not keywords anywhere else -- it's fair
+        to assume though that any POVRAY scene must have a camera and
+        lightsource."""
+        result = 0
+        if '#version' in text:
+            result += 0.05
+        if '#declare' in text:
+            result += 0.05
+        if 'camera' in text:
+            result += 0.05
+        if 'light_source' in text:
+            result += 0.1
+
+        return result
